(
s.waitForBoot{
	s.sync;
	~soundsPath = PathName.new("sounds/".resolveRelative);
	~soundsPath.filesDo({|path| path.fileNameWithoutExtension.postln });
	~buffs = SoundFile.collectIntoBuffers(~soundsPath.pathOnly ++ "*" );   //Load Sounds

	~specs = ( //Load custom control specifications
		rate1: ControlSpec(1, 2, \exp, 0.01, 0.1),
		rate2: ControlSpec(1, 3, \exp, 0.01, 0.1),
		rate3: ControlSpec(0.5, 1.5, \exp, 0.01, 0.1),
		rate4: ControlSpec(1, 1.5, \exp, 0.01, 0.1),
		rate5: ControlSpec(1, 1.5, \exp, 0.01, 0.1),
		attackall: ControlSpec(0.01, 0.1, \lin, 0.03),
		sustainall: ControlSpec(0.15, 0.8, \lin, 0.01, 0.1),
		releaseall: ControlSpec(0.25, 0.9, \lin, 0.01, 0.5),
		startPos: ControlSpec(0.1, 1.0, \lin, 0.01, 0.5);
	);

	s.bind({
		SynthDef(\grains, {|out= 0, bufnum, startPos= 0.5, rate= 1, attack=0.01,
			sustain= 0.6, release= 0.5, pan= 0, amp= 1|

			var play, env;

			env = Linen.kr(Impulse.kr(0), attack, sustain, release, doneAction:2);

			play = BufRd.ar(
				numChannels:1,
				bufnum:bufnum,
				phase:LFSaw.ar(BufDur.ir(bufnum).reciprocal).range *
				BufFrames.ir(bufnum) * rate * BufRateScale.ir(bufnum)
			);

			OffsetOut.ar( out, Pan2.ar(
				in:play * env,
				pos:pan,
				level:amp.lag(0.3)
			)
			)
		}).add;
	}.fork
	);

	//continuous controls//
	~pitch = 0.1;
	~roll = 0.1;
	~yaw = 0.1;
	~amp = 0.6;

	//static controls (Do not change, unless you know what you are doing)//
	~int1 = 1;
	~int2 = 2;
	~duration = 2.0;
	~gesture2 = true;
	~gesture3 = true;
	~gesture4 = true;

	//Load the patterns
	~p = Pspawner({ |sp|

		f = sp;
		~a = f.par(
			PfadeIn(
				Pbind(\instrument, \grains,
					\amp, Prand([0.9, 1.0], inf) * Pfunc{~amp},
					\pan, Pwhite(-1, 1, inf),
					\bufnum, ~buffs[8],
					\rate,  Pgbrown(Pfunc{~int1}, Pfunc{~int2}, 2.0, inf) * Pfunc { ~specs[\rate1].map(~pitch) },
					\startPos, Pwhite(0.1, 1.0, inf) * Pfunc { ~specs[\startPos].map(~roll) },
					\attack, Pbeta(0.01, 0.2, 5, 2 ) * Pfunc { ~specs[\attackall].map(~roll).max(0.03) },
					\sustain, max(0.003, Pwhite(0.9, 1.0, inf) * Pfunc { ~specs[\sustainall].map(~yaw).max(0.1) }),
					\release, Pgbrown(0.2, 2.0, 0.1) * Pfunc { ~specs [\releaseall].map (~yaw).max(0.3) },
					\dur, Pgbrown (0.01, 0.1, Pfunc { ~yaw }, inf)
				).asStream,
				fadeTime:3
			)
		);

		f.seq(Pbind(\type, Pif(Pfunc({ ~gesture2 }), \rest), \dur, 0.125));

		~b = f.par(
			PfadeIn(
				Pbind(\instrument, \grains,
					\amp,  Prand([(0.8..1.0)], inf) * Pfunc{~amp},
					\pan, Pwhite(-1, 1, inf),
					\bufnum, ~buffs[~buffs.size.rand, ~buff.size.rand, ~buffs.size.rand],
					\rate,  Pgbrown (Pbeta(10, 20, 0.2, 0.2, inf), Pbeta(10, 20, 2.0, 2.0, inf), Pbeta(1.0, 0.2, 0.1, 0.1, inf), inf) * Pfunc { ~specs[\rate2].map(~pitch) },
					\startPos, Pwhite(0.1, 1.0, inf) * Pfunc { ~specs[\startPos].map(~roll) },
					\attack, Pbeta(0.01, 0.2, 5, 2 ) * Pfunc { ~specs[\attackall].map(~roll).max(0.03) },
					\sustain, max(0.003, Pwhite(0.7, 1.0, inf) * Pfunc { ~specs[\sustainall].map(~yaw).max(0.1) }),
					\release, Pgbrown(0.1, 1.0, 0.125) * Pfunc { ~specs [\releaseall].map (~yaw).max(0.3) },
					\dur, Pgbrown (0.01, 0.1, Pfunc {~duration}, inf )
				).asStream,
				fadeTime:3
			)
		);

		f.seq(Pbind(\type, Pif(Pfunc({ ~gesture3 }), \rest), \dur, 0.125));

		~c = f.par(
			PfadeIn(
				Pbind(\instrument, \grains,
					\amp, Prand([(0.8..1.0)], inf) * Pfunc{~amp},
					\pan, Prand([-1, 1], inf),
					\bufnum, ~buffs[~buffs.size.rand, ~buff.size.rand, ~buffs.size.rand],
					\rate, Ptuple([
						Pseg(Pseq([10, 20], inf), 10),
						Pseg(Pseq([40, 60], inf), 10),
						Pseg(Pseq([80, 20], inf), 10)
					], inf) * Pfunc { ~specs[\rate3].map( ~pitch ) },
					\release, Pgbrown(0.2, 2.0, 0.1) * Pfunc { ~specs [\releaseall].map (~roll).max(0.3) } * Pfunc { ~yaw },
					\dur, Prand([0.1, 0.05, 0.03, 0.05, 0.07], inf)
				).asStream,
				fadeTime:3
			)
		);

		f.seq(Pbind(\type, Pif(Pfunc({ ~gesture4 }), \rest), \dur, 0.125));

		~d = f.par(
			PfadeIn(
				Pbind(\instrument, \grains,
					\amp, Prand([(0.6..1.0)], inf) * Pfunc{~amp},
					\pan, Prand([-1, 1], inf),
					\bufnum, Pxrand([ ~buffs[0], ~buffs[7] ], inf),
					\rate, Pgbrown(-1.0, 1, 0.125, inf).next * Pfunc { ~specs[\rate4].map(~pitch) } ,
					\attack,   Pwhite (0.01, 0.09,  inf ) * Pfunc { ~specs[\attackall].map(~roll).max(0.03) },
					\sustain, max(0.003, Pwhite(0.6, 1.0, inf) * Pfunc { ~specs[\sustainall].map(~roll).max(0.1) }),
					\release,  Pgbrown(0.1, 0.2, 0.01) * Pfunc { ~specs [\releaseall].map (~yaw).max(0.3) },
					\dur,  Pseq([0.01, 0.1], inf)
				).asStream,
				fadeTime:3
			)
		);
	});

	//Wiimote controls//
	OSCFunc({|msg|  ~pitch = msg[1] }, '/wii/1/accel/pry/0');
	OSCFunc({|msg|  ~roll = msg[1] }, '/wii/1/accel/pry/1');
	OSCFunc({|msg|  ~yaw = msg[1] }, '/wii/1/accel/pry/2');
	OSCFunc({|msg|  ~amp = msg[1] }, '/wii/1/accel/pry/3');
	OSCFunc({|msg| if(msg[1] == 1) { ~gesture2 = false; msg.postln; } }, '/wii/1/button/Up');
	OSCFunc({|msg| if(msg[1] == 1) { ~gesture3 = false; msg.postln;  } }, '/wii/1/button/Right');
	OSCFunc({|msg| if(msg[1] == 1) { ~gesture4 = false; msg.postln;  } }, '/wii/1/button/Down');
	OSCFunc({|msg| if(msg[1] == 1) { x = ~p.play; msg.postln; } }, '/wii/1/button/Plus');
	OSCFunc({|msg| if(msg[1] == 1) {  x.stop; msg.postln; }; }, '/wii/1/button/Minus');
});